/*!
    \file   df_lua_api.h
    \author Bernd Ottmann
    \date   16.09.2015
    \brief  Deklarationen der Funktionen für die Arbeit mit dem Lua-Interpreter.
*/
#ifndef DF_LUA_API_H
#define DF_LUA_API_H

#include "df_api.h"
#include "../../Shared/Common/AEErrorDef.h"

#if (DF_API_VERSION >= 0x04030600)

#ifndef MAX_CHANNELID
#define MAX_CHANNELID 401
#endif

#ifndef GLOBAL_CHANNELID
#define GLOBAL_CHANNELID (MAX_CHANNELID - 1)
#endif

#include <lua.hpp>
#include <string>

struct LuaChannel
{
    int id;
    int errorNumber;
};

#define checkchannel( L ) \
    reinterpret_cast<LuaChannel *>( luaL_checkudata( L, 1, "datafox.channel" ) );

#define push_globalErrorNumber( errorNumber, res ) \
{\
    lua_pushinteger( L, errorNumber );\
    std::string errorText( 255, '\0' );\
    DFCGetErrorText( GLOBAL_CHANNELID, errorNumber, 0, const_cast<char *>( errorText.data() ), errorText.size() );\
    lua_pushlstring( L, errorText.data(), errorText.find_first_of( '\0' ) );\
    res += 2;\
}

#define push_errorNumber( lc, res ) \
{\
    lua_pushinteger( L, lc->errorNumber );\
    std::string errorText( 255, '\0' );\
    DFCGetErrorText( lc->id, lc->errorNumber, 0, const_cast<char *>( errorText.data() ), errorText.size() );\
    lua_pushlstring( L, errorText.data(), errorText.find_first_of( '\0' ) );\
    res += 2;\
}

int luaopen_dfcomdll( lua_State *L );

#endif

#endif
