/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#ifndef CHANNEL4_BUFFER_H
#define CHANNEL4_BUFFER_H

// defines fuer Setup
#include "../Common/aes.h"

// Geraetedateiarchiv einbinden
#include "../File/filedevicezip.h"

// Studiodatei einbinden
#include "../File/filestudio.h"

// Datensatzgrosze fuer Zutrittslisten (Version 2) definieren
#define ENTRANCE2_RECORD_SIZE_0    33
#define ENTRANCE2_RECORD_SIZE_1    64
#define ENTRANCE2_RECORD_SIZE_2    20
#define ENTRANCE2_RECORD_SIZE_3    25
#define ENTRANCE2_RECORD_SIZE_4    21
#define ENTRANCE2_RECORD_SIZE_5    17
#define ENTRANCE2_RECORD_SIZE_6    21
#define ENTRANCE2_RECORD_SIZE_7    39

class Channel4Buffer
{
public:
    Channel4Buffer();
    virtual ~Channel4Buffer();

    void ChangeOrderLong(unsigned char *Buf);
    unsigned int ChangeOrderULong(unsigned int val);
    void ULongToBinMSBFirst(unsigned char *Buf,unsigned int Val);
    void ChangeOrderInt(unsigned char *Buf);

public:
    unsigned int StartDatensatzB, EndDatensatzB;
    int MaxListB;
    int MaxDatB;

    StructDatensatzBeschreibung DatensatzBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
    StructDatensatzFeld DatensatzFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

    StructDatensatzBeschreibung ListenBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
    StructDatensatzFeld ListenFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

    unsigned int m_entrance2Pointer[MAX_DATENSATZ_BESCHREIBUNG];
    StructTable2 m_entrance2Table[MAX_DATENSATZ_BESCHREIBUNG];
    StructTable2Field m_entrance2TableField[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD];

public:
    void reset();
    unsigned int GetMDEEKFlashSize_0400();
    unsigned int GetMDEEKFlashSize();
    unsigned int GetEntranceEKFlashSize();
    unsigned int GetEntrance2EKFlashSize();
    void AdjustExtendedMemPointer();
    void PrepareEntrance2EK(unsigned int offs, unsigned char *buf);
    unsigned int GetFlashSetupSize();


    // Speicher fuer Listendaten
    bool m_listSortCheck[MAX_DATENSATZ_BESCHREIBUNG];
    unsigned int m_listSize[MAX_DATENSATZ_BESCHREIBUNG];
    unsigned char m_ucListRecordSize[MAX_DATENSATZ_BESCHREIBUNG];
    unsigned char *m_pucListMem[MAX_DATENSATZ_BESCHREIBUNG];

    void ClearListBuffer();
    int MakeListe(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);

    // Sortieren der Datensaetze ueber ASCII - Feld. Offs ist Offset in Datensatz auf Feldbeginn.
    // Allgemein gültiger Sortieralgorithmus
    void QuicksortList(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs, int lLeftRecordNum, int lRightRecordNum);
    int QuicksortListPartition(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs, int lLeftRecordNum, int lRightRecordNum);
    void QuicksortListSwap(unsigned char *pucMem, unsigned char ucRecordSize, int lLeftRecordNum, int lRightRecordNum);

    static bool comparing(char*, char*);
    void SortList(unsigned char *pucMem, unsigned char ucRecordSize, unsigned char ucOffs);

    // Speicher fuer Zutrittskontrolle (Version 2) - Listendaten
    bool m_entrance2SortCheck[8];
    unsigned int m_ulEntrance2ListSize[8];
    unsigned char m_ucEntrance2RecordSize[8];
    unsigned char *m_pucEntrance2ListMem[8];

    void ClearEntrance2ListBuffer(int iListNum = -1);

    int CheckEntrance2List(unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned int size);
    int CheckEntrance2Dec(char *buf, int min, int max);
    int CheckEntrance2TM(char *buf);
    int CheckEntrance2Date(char *buf);
    int CheckEntrance2Time(char *buf);
    int CheckEntrance2Weekday(char *buf);
    int MakeEntrance2List(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);
    int ImportEntrance2List(unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned int size);

    // Speicher fuer Timeboy - Listendaten
    unsigned int m_ulTimeboyListSize[MAX_DATENSATZ_BESCHREIBUNG][MAX_DATENSATZ_BESCHREIBUNG];
    unsigned char *m_pucTimeboyListMem[MAX_DATENSATZ_BESCHREIBUNG][MAX_DATENSATZ_BESCHREIBUNG];
    int m_timeboyFlags[MAX_DATENSATZ_BESCHREIBUNG][MAX_DATENSATZ_BESCHREIBUNG];

    void ClearTimeboyListBuffer(unsigned char id = 0xFF, unsigned char num = 0xFF);
    int MakeTimeboyList(unsigned char id, unsigned char num, unsigned short recordCount, unsigned char *buf, unsigned int size, int flags, int *error);

    bool WriteListToBinFile( unsigned char listNum, const char *filePath );
    bool WriteListToTextFile( unsigned char listNum, const char *filePath );
    bool WriteListEntrance2ToBinFile( unsigned char listNum, const char *filePath );
    bool WriteListEntrance2ToTextFile( unsigned char listNum, const char *filePath );
    bool WriteListTimeboyToBinFile( unsigned char listId, unsigned char listNum, const char *filePath );

    // fuer Setupdaten
    FileStudio m_file;

    int GetTableCount(unsigned char ucList = 0);

    int GetTableRecord(unsigned char ucNum,unsigned char *szName, 
        unsigned char *pucFieldCount, unsigned char *pucVariable = NULL, 
        unsigned char ucList = 0);

    int GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
        unsigned char *szName, unsigned char *pucType, 
        unsigned char *pucLength, unsigned char ucList = 0);

    void DefaultIOModule(StructIOModule *pIOModule);

    // Dateiarchiv fuer Geraetedaten (Firmware) verwalten
    FileDeviceZip m_fileDevice;

public:
    // Zutrittskontrolllisten (Version 2) aufsetzen.
    void Entrance2Default();
    int GetTableRecordEntrance2(unsigned char ucNum,unsigned char *szName, 
        unsigned char *pucFieldCount);
    int GetTableFieldEntrance2(unsigned char ucTableNum, unsigned char ucFieldNum, 
        unsigned char *szName, unsigned char *pucType, 
        unsigned char *pucLength);
};

#endif
