/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "Channel4ModuleApplication.h"
#include "../Common/DFSoftwareVersion.h"
#include <stdio.h>
#include <boost/algorithm/string.hpp>
#include <regex>

// ##################################################################
/**
 * @brief Initialisieren der Membervariablen.
 *
 * @param id Nummer der Kanalklasse fuer z.B. Trace.
 */
Channel4ModuleApplication::Channel4ModuleApplication(short channelID, short deviceID) : Channel4ModuleSystem(channelID, deviceID) 
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, channelID, deviceID);

    // Anzahl anstehender Datensaetze
    srand((unsigned int)time(NULL));
}

// ##################################################################
/**
 * @brief Destruktor.
 *
 */
Channel4ModuleApplication::~Channel4ModuleApplication() 
{
    while(m_records.size()) 
    {
        m_records.pop();
    }
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void Channel4ModuleApplication::getErrorModule(short module, short error, std::string &text)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, module, error, text.c_str());

    // Ist IV_MODULE_APPLICATION ?
    if (module == IV_MODULE_APPLICATION)
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ModuleSystem::getErrorModule(module, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 */
void Channel4ModuleApplication::getErrorFunction(short module, short func, short error, std::string &text)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str());

    if (module == IV_MODULE_APPLICATION)
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if (error == 28) 
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %i ): Korrektes Passwort für die Kommunikation erforderlich.") % error );
            return;
        }
        else if (error == 29) 
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %i ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt.") % error );
            return;
        }

        switch (func)
        {
        case IV_FUNCTION_CLEAR_RECORDS:
        case IV_FUNCTION_GET_RECORD:
        case IV_FUNCTION_SET_RECORD:
            switch(error) {
            case 1:text = boost::str( boost::format( "( %i ): ") % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error ); break;
            }
            break;
        case IV_FUNCTION_SHOW_MESSAGE:
            switch(error) {
            case 1:text = boost::str( boost::format( "( %i ): Nachrichtentext zu lang fuer die Anzeige.") % error ); break;
            case 2:text = boost::str( boost::format( "( %i ): Es wird bereits eine Nachricht angezeigt.") % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error ); break;
            }
            break;
        case IV_FUNCTION_SET_VARIABLE:
        case IV_FUNCTION_GET_VARIABLE:
            switch(error) {
            case 1:text = boost::str( boost::format( "( %i ): Der Majorbestandteil des Variablennamens ist unbekannt.") % error ); break;
            case 2:text = boost::str( boost::format( "( %i ): Der Minorbestandteil des Variablennamens ist unbekannt.") % error ); break;
            case 3:text = boost::str( boost::format( "( %i ): Der zu setzende Variablenwert ist zu lang.") % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error ); break;
            }
            break;
        case IV_FUNCTION_SHOW_INFO:
            switch(error) {
            case 1:text = boost::str( boost::format( "( %i ): Nachrichtentext zu lang fuer die Anzeige.") % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error ); break;
            }
            break;
        case IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND:
        case IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND_OK:
            switch(error) {
            case 1: text = boost::str( boost::format( "( %i ): ZK nicht aktiv.") % error ); break;
            case 2: text = boost::str( boost::format( "( %i ): Parameter Command außerhalb seines gültigen Bereichs.") % error ); break;
            case 3: text = boost::str( boost::format( "( %i ): Parameter Master ID außerhalb seines gültigen Bereichs.") % error ); break;
            case 4: text = boost::str( boost::format( "( %i ): Parameter KnobId außerhalb seines gültigen Bereichs.") % error ); break;
            case 5: text = boost::str( boost::format( "( %i ): Speicherfehler.") % error ); break;
            case 6: text = boost::str( boost::format( "( %i ): Kommunikationsfehler mit dem Funkknoten.") % error ); break;
            case 7: text = boost::str( boost::format( "( %i ): Die Ausführung des Befehls erfolgte, doch der Schließlinder konnte nicht entfernt werden.") % error ); break;
            case 8: text = boost::str( boost::format( "( %i ): Die Ausführung des Befehls erfolgte, doch der Schließlinder konnte nicht eingelernt werden.") % error ); break;
            case 9: text = boost::str( boost::format( "( %i ): Die Ausführung des Befehls erfolgte, doch der Schließlinder konnte nicht eingelernt werden, weil er einen Produktschutz hat.") % error ); break;
            default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error ); break;
            }
            break;
        default: text = boost::str( boost::format( "( %i ): Nicht eingetragener Fehlerblock für Funktion!") % error ); break;
        }
        
        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ModuleSystem::getErrorFunction(module, func, error, text);
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::remoteCall(short module, short func, unsigned char *data, int &count)
{
    if (module == IV_MODULE_APPLICATION)
    {
        unsigned char status;
        switch (func)
        {
        case IV_FUNCTION_CLEAR_RECORDS: return clearRecords(getDeviceId());
        case IV_FUNCTION_SET_RECORD: return setRecord(getDeviceId(), status);
        case IV_FUNCTION_GET_RECORD: return getRecord(getDeviceId(), data, count, status);
        case IV_FUNCTION_SHOW_MESSAGE: return showMessage(getDeviceId(), data[0], data[1], data[2], (char*)&data[3]);
        case IV_FUNCTION_SET_VARIABLE: return setVariable(getDeviceId(), (char*)&data[0], (char*)&data[strlen((char*)&data[0]) + 1]);
        case IV_FUNCTION_GET_VARIABLE: return getVariable(getDeviceId(), (char*)&data[0], (char*)&data[strlen((char*)&data[0]) + 1]);
        case IV_FUNCTION_SHOW_INFO: return showInfo(getDeviceId(), data[0], data[1], data[2], data[3], *(short*)&data[6], (char*)&data[8]);
        case IV_FUNCTION_ENTRANCE2_IDENTIFICATION: 
            {
            char *id, *pin;
            char status[64]; 
            int statusLength;
            id = (char*)&data[strlen((char*)&data[0]) + 1];
            pin = &id[strlen(id) + 1];
            return Entrance2Identification(getDeviceId(), (char*)&data[0], id, pin, status, statusLength); 
            }
        case IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK: return Entrance2IdentificationOK(getDeviceId());
        case IV_FUNCTION_NOTIFY: return Notify(getDeviceId(), data[0], *(short*)&data[1], (char*)&data[3], (char*)&data[3 + strlen((char*)&data[3])]);
        case IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION_OK: return accessControlIdentificationOK( getDeviceId() );
        case IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND_OK: return accessControlKnobCommandOK( getDeviceId() );
        case IV_FUNCTION_CUSTOM_COMMAND_OK: return customCommandOK( getDeviceId() );
		// Funktion nicht definiert
        default: return 0x0200; 
        }

    }

    return Channel4ModuleSystem::remoteCall(module, func, data, count);
}

//#########################################################
//##################### APPLICATION #######################
//#########################################################

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::clearRecords(short deviceID)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        while (m_records.size()) 
        {
            m_records.pop();
        }

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_CLEAR_RECORDS, tmp, count)) != IV_ERROR_SUCCESS) 
        {
            
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
*/
int Channel4ModuleApplication::restoreRecords(short deviceID)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_RESTORE_RECORDS, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_RESTORE_RECORDS);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;

//		while (m_records.size()) 
//		{
//			m_records.pop();
//		}

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_RESTORE_RECORDS, tmp, count)) != IV_ERROR_SUCCESS) 
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::setRecord(short deviceID, unsigned char &status)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, status);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD);
            return _result;
        }

        // OK
        if (count > (int)sizeof(unsigned char)) 
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }
        
        switch (_packetData[0]) 
        {
        case 0:
        case 2:
            status = 0;
            break;

        case MB_ANSW_ACK_ON_RECORD:        // Quittung erhalten (fuer Online-Datensatz) 
        case MB_ANSW_TIMEOUT_ACK_RECORD:   // Quittung kam nach Timeout (fuer Online-Datensatz) 
            status = 1;
            break;

        case MB_ANSW_OLD_ACK_ON_RECORD:    // wiederholt gesendete Quittung 
        case MB_ANSW_OLD_TIMEOUT_ACK_RECORD: // wiederholte Quittung nach Timeout 
            status = 2;
            break;

        case MB_ANSW_ACK_OFF_RECORD:       // Quittung erhalten (fuer Offline-Datensatz) 
            status = 3;
            break;

        case MB_ANSW_OLD_ACK_OFF_RECORD:   // wiederholt gesendete Quittung 
            status = 4;
            break;
        }
    } 
    else 
    {
        unsigned char data[1];
        
        if (m_records.size()) 
        {
            CRecord &record = m_records.front();

            switch (record.m_data[0]) 
            {
            case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
                data[0] = 0;
                break;

            case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
            case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
                data[0] = MB_ANSW_ACK_ON_RECORD;
                break;

            case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
            case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
                data[0] = MB_ANSW_ACK_OFF_RECORD;
                break;
            }
            
            count = sizeof(unsigned char);

            // Datensatz loeschen
            m_records.pop();
        } 
        else 
        {
            data[0] = 0;
            count = sizeof(unsigned char);
        }

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_RECORD, data, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::getRecord(short deviceID, unsigned char *buf, int &count, unsigned char &status)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x08X, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, buf, count, status);

    int i;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD);
            return _result;
        }

        // OK
        
        switch (_packetData[0]) 
        {
        case MB_ANSW_NO_RECORD:             // kein Datensatz vorhanden
            status = 0;
            break;

        case MB_ANSW_ONLINE_RECORD:         // Online Datensatz 
            status = 1;
            break;

        case MB_ANSW_OLD_ONLINE_RECORD:     // wiederholt gesendeter Online Datensatz 
            status = 2;
            break;

        case MB_ANSW_OFFLINE_RECORD:        // Offline Datensatz 
            status = 3;
            break;

        case MB_ANSW_OLD_OFFLINE_RECORD:    // wiederholt gesendeter Offline Datensatz
            status = 4;
            break;
        }
        
        if (((status == 0) && (count > 1)) || (count > 243)) 
        {	
            return -IV_ERROR_PROCESSING_DLL;
        }


        for (i=0; i < count; i++) 
        {	
            buf[i] = _packetData[i];
        }
    } 
    else 
    {
        unsigned char data[256];
        count = 0;
        
        if (m_records.size()) 
        {
            CRecord &record = m_records.front();

            // Daten kopieren
            data[0] = record.m_status;
            memcpy(&data[1], record.m_data, record.m_length);
            count = 1 + record.m_length;
        } 
        else 
        {
            data[count++] = MB_ANSW_NO_RECORD;
        }

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_RECORD, data, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::showMessage(short deviceID, unsigned char time, unsigned char action, unsigned char audio, char *text)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceID, time, action, audio, text);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        _packetData[count] = time;
        count += sizeof(unsigned char);
        _packetData[count] = action;
        count += sizeof(unsigned char);
        _packetData[count] = audio;
        count += sizeof(unsigned char);

        // Namen uebernehmen
        count += strlen( strcpy( (char*)&_packetData[count], text ) ) + 1;
        
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_MESSAGE, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):INFO(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, text);
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::setVariable(short deviceID, char *name, char *val)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceID, name, val);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Namen uebernehmen
        count = strlen( strcpy( (char*)_packetData, name ) ) + 1;
        // Wert uebernehmen
        count += strlen( strcpy( (char*)&_packetData[count], val ) ) + 1;
        
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_VARIABLE, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):INFO(1): %s [ %s = %s ]\n", __FILE__, __LINE__, __func__, name, val);
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::getVariable(short deviceID, char *name, char *val)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceID, name, val);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Namen uebernehmen
        count = strlen( strcpy( (char*)_packetData, name ) ) + 1;
        
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE);
            return _result;
        }

        // Name+Wert als Antwortdaten erwartet
        if (count > (17 + 81)) 
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Namen uebergehen
        count = (int)strlen(name) + 1;

        // OK
        strcpy(val, (char*)&_packetData[count]);	
    
    } 
    else 
    {
        unsigned char tmp[17 + 81];
        memset(tmp, 0, sizeof(tmp));
        count = strlen( strcpy( (char*)tmp, name ) ) + 2;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_GET_VARIABLE, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):INFO(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, name);
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::showInfo(short deviceID, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, char *text)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i, %i, 0x%04X, 0x%08X ]\n", __FILE__, __LINE__, __func__, deviceID, maschine, time, font, audio, flags, text);

    int count = 0;
// Maschinennummer (1 Byte)
// Time ist die Anzeigedauer. (1 Byte) (0 im Hindergrund uebernehmen, -1 staendig anzeigen)
// Schriftgroessennummer (1 Byte)
// Akkustisches Signal (1 Byte)
// 16bit reserve (2 Byte)
// Flags z.b. 0x0001 (2 Byte) fuer Antiproportionale Schrift
// Infotext
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        _packetData[count] = maschine;
        count += sizeof(unsigned char);
        _packetData[count] = time;
        count += sizeof(unsigned char);
        _packetData[count] = font;
        count += sizeof(unsigned char);
        _packetData[count] = audio;
        count += sizeof(unsigned char);
        *(short*)&_packetData[count] = 0;
        count += sizeof(short);
        *(short*)&_packetData[count] = flags;
        count += sizeof(short);

        // Namen uebernehmen
        count += strlen( strcpy( (char*)&_packetData[count], text ) ) + 1;
        
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SHOW_INFO, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):INFO(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, text);
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::Entrance2Identification(short deviceID, char *TM, char *Id, char *Pin, char *status, int &statusLength)
{
    trace("%s(%i):INFO(3): %s [ %i, %s, %s, %s ]\n", __FILE__, __LINE__, __func__, deviceID, TM, Id, Pin);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        std::string TM_( TM );
        std::string Id_( Id );
        std::string Pin_( Pin );
        std::string regTM("[0-3]\\d[01]");
        std::string regId("\\w{1,20}");
        std::string regPin("\\d{0,8}");

        // Format pruefen
        if ( !std::regex_match( TM_, std::regex( regTM ) ) 
            || !std::regex_match( Id_, std::regex( regId ) ) 
            || !std::regex_match( Pin_, std::regex( regPin ) ) )
            return -IV_ERROR_FUNCTION_PARAMETER;

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Namen uebernehmen
        count = strlen( strcpy( (char*)&_packetData[count], TM ) ) + 1;
        count += strlen( strcpy( (char*)&_packetData[count], Id ) ) + 1;
        count += strlen( strcpy( (char*)&_packetData[count], Pin ) ) + 1;
        
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        count = 0;

        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK);
            return _result;
        }

        if (count == 0) 
        {
            trace("%s(%i):WARNING(1): %s [ Aktiver Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Wert von Status uebernehmen
        int i;
        for (i=0; (i < count) && (i < statusLength); i++) 
        {
            status[i] = _packetData[i];
        }
        // Uebergebene Laenge mitteilen
        statusLength = i - 1;
    } 
    else 
    {
        unsigned char tmp[64];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        // Aktuellen Status als aktive Antwort senden
        count = strlen( strcpy( (char*)tmp, "Keiner" ) ) + 1;

        // Aktive Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::Entrance2IdentificationOK(short deviceID)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        trace("%s(%i):ERROR(1): %s \n", __FILE__, __LINE__, __func__);
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_IDENTIFICATION_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):ERROR(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, _result);
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
*/
int Channel4ModuleApplication::Entrance2OnlineAction(short deviceID, char *TM, unsigned int mask, unsigned char type, unsigned char duration)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x08X, 0x%08X, %u, %u ]\n", __FILE__, __LINE__, __func__, deviceID, TM, mask, type, duration);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        std::string TM_( TM );
        std::string regTM("[0-3]\\d[01]");

        if ( !std::regex_match( TM_, std::regex( regTM ) ) ) 
            return -IV_ERROR_FUNCTION_PARAMETER;

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Nummer des Tuermodul uebernehmen
        count = strlen( strcpy( (char*)_packetData, TM ) ) + 1;
        // Weiteren Werte uebernehmen
        *(unsigned int*)&_packetData[count] = mask;
        count += sizeof(unsigned int);
        _packetData[count] = type;
        count += sizeof(unsigned char);
        _packetData[count] = duration;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_ONLINE_ACTION, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_ONLINE_ACTION);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ENTRANCE2_ONLINE_ACTION, tmp, count)) != IV_ERROR_SUCCESS) 
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):INFO(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, TM);
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::Notify(short deviceID, unsigned char type, int priority, char *timestamp, char *params)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %s ]\n", __FILE__, __LINE__, __func__, deviceID, type, priority, params);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Benachrichtigungstyp uebernehmen
        _packetData[count] = type;
        count += sizeof(unsigned char);
        // Prioritaet eintragen
        *(short*)&_packetData[count] = priority;
        count += sizeof(short);
        // Zeitstempel
        if (timestamp == NULL) 
        {
            time_t tm_time;
            struct tm *ptm;
            tm_time = time(NULL);
            ptm = localtime(&tm_time);
            count += sprintf((char*)&_packetData[count], "%02i%02i-%02i-%02i %02i:%02i:%02i:%03i", 
                (ptm->tm_year + 1900) / 100,
                (ptm->tm_year + 1900) % 100,
                ptm->tm_mon + 1,
                ptm->tm_mday,
                ptm->tm_hour,
                ptm->tm_min,
                ptm->tm_sec, 0) + 1;
        }
        else 
        {
            count += strlen( strcpy( (char*)&_packetData[count], timestamp ) ) + 1;
        }
        // Leerstring
        count += strlen( strcpy( (char*)&_packetData[count], params ) ) + 1;

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY, _packetData, count)) != IV_ERROR_SUCCESS)
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY);
            return _result;
        }

        // OK
        if (count > (int)sizeof(int)) 
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }
    }
    else 
    {
        unsigned char tmp[4];
        count = sizeof(tmp);
        tmp[0] = 1;
        
        switch (type) 
        {
        case IV_VALUE_NOTIFY_RECORD: 
            {
            CNotifyRecord nr;

            // Hat das Kanalobjekt den Aktiven Zustand?
            if (getActiveState() < is_active) 
            {
                // Funktion derzeit nicht ausführbar.
                return 0x0001;
            }
            //TRACE("ActiveState: %i\n", GetActiveState());

            if (m_CBNotifyRecord) 
            {
                int pos = 0;

                // Aktuelle Kanalnummer eintragen
                nr.channelID = getId();

                // Aktuelle RemoteBusnummer eintragen
                nr.deviceID = getDeviceIdRemote();

                // Meldungstypen uebernehmen
                nr.type = type;
                pos += sizeof(unsigned char);

                // Meldungsprioritaet uebernehmen
                nr.priority = priority;
                pos += sizeof(unsigned short);

                // Zeitstempel der Meldungserzeugung uebernehmen
                pos += strlen( strcpy( nr.timestamp, timestamp ) );

                // Aktuellen Zeitstempel fuer Nachrichtenannahme halten
                nr.dt = time(NULL);

                // Meldung in Warteschlange stellen.
                if (m_CBNotifyRecord(nr) != IV_ERROR_SUCCESS) 
                {
                    // Die Annahme der Aktiven Datensatzmeldung schlug fehl (ggf. nicht aktiv).
                    return 0x0002;
                }

                trace("%s(%i):INFO(1): %s [ %i ]\n", __FILE__, __LINE__, __func__, getId());
            } 
            else 
            {
                // Die Annahme der Aktiven Datensatzmeldung ist nicht aktiv.
                return 0x0002;
            }
            break;
            }

        default:
            trace("%s(%i):INFO(1): %s [ Unknown type %i ]\n", __FILE__, __LINE__, __func__, type);
            break;
        }


        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_NOTIFY, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::NotifyRecord(short deviceID, int priority)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, priority);

    return Notify(deviceID, IV_VALUE_NOTIFY_RECORD, priority, NULL, (char*)"");
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
* @param type 
0		Standardschrift(Sicherstellung der Abwärtskompatibilität)
1		16 Pixel (7Anzeigezeilen)
2		16 Pixel,feste breite(7Zeilen)
3		19 Pixel(6Anzeigezeilen)
4		19 Pixel,feste breite(6Zeilen)
5		21 Pixel(5Anzeigezeilen)
6		21 Pixel,feste breite(5Zeilen)
*/
int Channel4ModuleApplication::SetTypeFont(short deviceID, unsigned char type)
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, type);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Typewert uebernehmen
        count = sizeof(unsigned char);
        // Wert uebernehmen
        *_packetData = type;

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_TYPE_FONT, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_SET_TYPE_FONT);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_SET_TYPE_FONT, tmp, count)) != IV_ERROR_SUCCESS) 
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die Anzahl der vorliegenden Datensaetze oder verwendetes Speichervolumen.
*
* @param [in]     deviceID Anzusprechende Geraetenummer.
* @param [out]    number Parameter der die Anzahl Datensaetze erhaelt. Er wird auf -1 gesetzt, wenn der Wert nicht ermittelt werden konnte.
* @param [out]    memoryUsage Parameter der das Datenvolumen erhaelt. Er wird auf -1 gesetzt, wenn der Wert nicht ermittelt werden konnte.
* @param [out]    memorySize Parameter der das Gesamtvolumen fuer Datensaetze erhaelt. Er wird auf -1 gesetzt, wenn der Wert nicht ermittelt werden konnte.
*/
int Channel4ModuleApplication::recordVolume(short deviceID, int &number, int &memoryUsage, int &memorySize)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i ]\n", __FILE__, __LINE__, __func__, deviceID, number, memoryUsage, memorySize);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Zuerst wird davon ausgegangen, dass beide Werte nicht ermittelt werden koennen.
        number = -1;
        memoryUsage = -1;
        memorySize = -1;

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_RECORD_VOLUME, _packetData, count)) == IV_ERROR_SUCCESS)
        {	
            // Antwortdaten erwartet
            if (count != (int)sizeof(unsigned int)*3)
            {		
                trace("%s(%i):WARNING(1): %s [ Ermittlung Anzahl Datensaetze und Speicher mit %i Byte, ungleich erwartete %i Byte ]\n", __FILE__, __LINE__, __func__, count - 3 * sizeof(int), 3 * sizeof(int));
            }

            // Anzahl der Datensaetze uebernehmen.
            number = *(int*)_packetData;
            if (count > (int)sizeof(int))
                memoryUsage = *(int*)&_packetData[sizeof(int)];
            if (count > (int)sizeof(int)*2)
                memorySize = *(int*)&_packetData[sizeof(int)*2];

            return _result;
        }

        // Nur der Modulfehler -Funktion unbekannt- wird akzeptiert.
        if (_result < 0 || (_result > 0 && ((_result >> 8) & 0xFF) != 2)) {

            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_RECORD_VOLUME);
            return _result;

        }

        // Wird die Funktion nicht unterstuetzt, dann wird versucht ueber die Systemvariable SYSTEM.STATUS die Information
        // zu ermitteln.
        char status[IV_PACKET_LENGTH_DATA];
        if ((_result = getSystemVariable(getDeviceIdRemote(), (char*)"SYSTEM.STATUS", status)) != IV_ERROR_SUCCESS) {

            return _result;
        }

        // Im Status wird nach "RECORDS:" gesucht.
        // 1. Es liegt nicht vor.
        // 2. Es liegt mit der Angabe des benoetigten Datenvolumens vor.
        // 3. Es liegt mit der Angabe der Datensatzanzahl und des Datenvolumens vor.
        // 4. Bei aktueller Berechnung (Zwecks MMC) kann auch "RECORDS: ?" kommen, dann wird noch ermittelt.
        char *pos, *end, *res;
        int val1 = -1; // Haelt den ersten ermittelten Wert.
        int val2 = -1; // Haelt den zweiten ermittelten Wert
        pos = strstr(status, "RECORDS:");
        if (pos == NULL) {

            // Die Informationen konnten nicht ermittelt werden.
            return _result;
        }
        pos += 9;
        end = strstr(pos, "\r");
        if (end == NULL) end = &status[strlen(status)];
        if ((end - pos) > 20 || (pos - status) < 20) {

            // Die Informationen konnten nicht ausgewertet werden.
            return _result;
        }
        if (*pos == '?') {

            // Die Information wird intern noch ermittelt und steht zur Zeit noch nicht zur Verfuegung.
            return _result;
        }
        // Die vorderen 20 Zeichen des status-Puffers fuer halten der Werte verwenden.
        res = status;
        memset(res, 0, 20);
        while (pos != end) {

            if (isdigit(*pos)) {
                *res = *pos;
                res++;
            } else if (*pos == ' ') {
                // Wert 1 oder 2 fuellen.
                if (val1 == -1) {
                    val1 = atoi(status);
                    res = status;
                    memset(res, 0, 20);
                } else if (val2 == -1) {
                    val2 = atoi(status);
                    break;
                }
            } else if (*pos == 'b') {
                // Es liegt nur der Erste Wert als Volumeangabe vor.
                break;
            }

            // Naechstes Zeichen.
            pos++;
        }

        if (val2 == -1) {
            // Es liegt die Angabe in Datenvolumen vor.
            memoryUsage = val1;
        } else {
            // Es liegen beide Angaben vor. Datensatzanzahl und volumen.
            number = val1;
            memoryUsage = val2;
        }
    }
    else
    {		
        unsigned char tmp[64];
        count = sizeof(int)*3;
        *(int*)tmp = '?';
        *(int*)&tmp[sizeof(int)] = 0;
        *(int*)&tmp[sizeof(int)*2] = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_RECORD_VOLUME, tmp, count)) != IV_ERROR_SUCCESS)
        {			
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::accessControlIdentification(short deviceID, int masterId, char *TM, char *Id, char *Pin, char *status, int &statusLength)
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %s, %s, %s ]\n", __FILE__, __LINE__, __func__, deviceID, masterId, TM, Id, Pin);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        int res;
        std::array<char, IV_PACKET_LENGTH_DATA> buffer;
        std::string TM_( TM );
        std::string Id_( Id );
        std::string Pin_( Pin );
        std::string regTM("[0-3]\\d[01]");
        std::string regId("\\w{1,20}");
        std::string regPin("\\d{0,8}");

        // Format pruefen
        if ( !std::regex_match( TM_, std::regex( regTM ) ) 
            || !std::regex_match( Id_, std::regex( regId ) ) 
            || !std::regex_match( Pin_, std::regex( regPin ) ) )
            return -IV_ERROR_FUNCTION_PARAMETER;

        // Angeggebene MasterId gegen die eingestellte prüfen.

        // Systemvariable ACCESS.MASTERID ermitteln.
        if ( (res = getSystemVariable( deviceID, "ACCESS.MASTERID", buffer.data() )) != IV_ERROR_SUCCESS )
            return res;

        std::string currentMasterId( buffer.data() );
        // Die anzusprechende MasterId muss größer oder gleich der aktuell eingestellten sein.
        if ( masterId < std::stoi( currentMasterId ) )
            return -IV_ERROR_FUNCTION_PARAMETER;

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Namen uebernehmen
        *reinterpret_cast<unsigned short*>( _packetData ) = masterId;
        count = sizeof(unsigned short);
        count += strlen( strcpy( (char*)&_packetData[count], TM ) ) + 1;
        count += strlen( strcpy( (char*)&_packetData[count], Id ) ) + 1;
        count += strlen( strcpy( (char*)&_packetData[count], Pin ) ) + 1;
        
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        count = 0;

        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION_OK);
            return _result;
        }

        if (count == 0) 
        {
            trace("%s(%i):WARNING(1): %s [ Aktiver Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        // Wert von Status uebernehmen
        int i;
        for (i=0; (i < count) && (i < statusLength); i++) 
        {
            status[i] = _packetData[i];
        }
        // Uebergebene Laenge mitteilen
        statusLength = i - 1;
    } 
    else 
    {
        unsigned char tmp[64];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        // Aktuellen Status als aktive Antwort senden
        count = strlen( strcpy( (char*)tmp, "Keiner" ) ) + 1;

        // Aktive Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::accessControlIdentificationOK(short deviceID)
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        trace("%s(%i):ERROR(1): %s \n", __FILE__, __LINE__, __func__);
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_IDENTIFICATION_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):ERROR(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, _result);
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceID Anzusprechende Geraetenummer.
*/
int Channel4ModuleApplication::accessControlOnlineAction(short deviceID, int masterId, char *TM, unsigned int mask, unsigned char type, unsigned char duration)
{
    trace("%s(%i):INFO(3): %s [ %i, 0x08X, 0x%08X, %u, %u ]\n", __FILE__, __LINE__, __func__, deviceID, TM, mask, type, duration);

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        int res;
        std::array<char, IV_PACKET_LENGTH_DATA> buffer;
        std::string TM_( TM );
        std::string regTM("[0-3]\\d[01]");

        if ( !std::regex_match( TM_, std::regex( regTM ) ) ) 
            return -IV_ERROR_FUNCTION_PARAMETER;

        // Angeggebene MasterId gegen die eingestellte prüfen.

        // Systemvariable ACCESS.MASTERID ermitteln.
        if ( (res = getSystemVariable( deviceID, "ACCESS.MASTERID", buffer.data() )) != IV_ERROR_SUCCESS )
            return res;

        std::string currentMasterId( buffer.data() );
        // Die anzusprechende MasterId muss größer oder gleich der aktuell eingestellten sein.
        if ( masterId < std::stoi( currentMasterId ) )
            return -IV_ERROR_FUNCTION_PARAMETER;

        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // anzusprechende MasterId
        *reinterpret_cast<unsigned short*>( _packetData ) = masterId;
        count = sizeof(unsigned short);
        // Nummer des Tuermodul uebernehmen
        count += strlen( strcpy( (char*)&_packetData[count], TM ) ) + 1;
        // Weiteren Werte uebernehmen
        *(unsigned int*)&_packetData[count] = mask;
        count += sizeof(unsigned int);
        _packetData[count] = type;
        count += sizeof(unsigned char);
        _packetData[count] = duration;
        count += sizeof(unsigned char);

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_ONLINE_ACTION, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_ONLINE_ACTION);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count);
        }
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_ONLINE_ACTION, tmp, count)) != IV_ERROR_SUCCESS) 
        {
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):INFO(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, TM);
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::accessControlKnobCommand( short deviceID, unsigned char knobCommand, unsigned short masterId, unsigned char knobId, char *params, unsigned char *data, int *sizeData )
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i, %s ]\n", __FILE__, __LINE__, __func__, deviceID, knobCommand, masterId, knobId, params);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);

        // Namen uebernehmen
        *reinterpret_cast<unsigned char*>( &_packetData[count] ) = knobCommand;
        count = sizeof(unsigned char);
        *reinterpret_cast<unsigned short*>( &_packetData[count] ) = masterId;
        count += sizeof(unsigned short);
        *reinterpret_cast<unsigned char*>( &_packetData[count] ) = knobId;
        count += sizeof(unsigned char);
        count += strlen( strcpy( (char*)&_packetData[count], params ) ) + 1;
        
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND);
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        count = 0;

        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND_OK);
            return _result;
        }

        // Die Daten werden als eine Listen von Hexwerten mit \r getrennt geliefert.
        // Keine Antwortdaten erwartet
        if ( knobCommand == IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_LIST && count != 64 ) 
        {
            trace( "%s(%i):WARNING(1): %s [ Es wurden 64 Byte Antwort erwartet und %i Byte erhalten. ]\n", __FILE__, __LINE__, __func__, count );
        }
        
        if ( knobCommand == IV_VALUE_ACCESS_CONTROL_KNOB_COMMAND_LIST )
        {
            // Die Daten müssen entsprechend aufbereitet werden.
            auto swapByte = []( unsigned char *d, unsigned char *s )
            {
                unsigned char tmp = *d;
                *d = *s;
                *s = tmp;
            };

            auto toLSB = [=]( unsigned char *p ) 
            { 
                swapByte( &p[0], &p[3] );
                swapByte( &p[1], &p[2] );
                return *reinterpret_cast<int *>( p );
            };

            // Ggf. gelieferte Daten übernehmen.
            for ( int i = 0; i < (count / 4) && ((i + 1) * 4 + i) < *sizeData; ++i )
                sprintf( reinterpret_cast<char *>( &data[i * 9] ), "%08X%c", toLSB( &_packetData[i * 4] ), (i + 1) == (count / 4) ? '\0' : '\r' );

            *sizeData = count;
        }
        else
        {
            // Daten wie sie sind übernehmen.
            int i = 0;
            for ( ; i < count && i < *sizeData; ++i )
                data[i] = _packetData[i];

            *sizeData = i;
        }
    } 
    else 
    {
        unsigned char tmp[64];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        // Aktuellen Status als aktive Antwort senden
        count = strlen( strcpy( (char*)tmp, "Keiner" ) ) + 1;

        // Aktive Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::accessControlKnobCommandOK( short deviceID )
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        trace("%s(%i):ERROR(1): %s \n", __FILE__, __LINE__, __func__);
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_ACCESS_CONTROL_KNOB_COMMAND_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):ERROR(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, _result);
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::customCommand( short deviceID, unsigned int identification, unsigned int customCommand, char *data, int *sizeData )
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %i, %s ]\n", __FILE__, __LINE__, __func__, deviceID, customCommand);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID) 
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote(deviceID);
        
		*reinterpret_cast<unsigned int*>( &_packetData[count] ) = identification;
		count = sizeof(unsigned int);
		*reinterpret_cast<unsigned int*>( &_packetData[count] ) = customCommand;
		count += sizeof(unsigned int);

		*reinterpret_cast<unsigned int*>( &_packetData[count] ) = *sizeData;
		count += sizeof(unsigned int);

		for ( int i = 0; i < *sizeData; i ++ )
		{
			*reinterpret_cast<char*>( &_packetData[count] ) =  data[i];
			count += sizeof( char );
		}

        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_CUSTOM_COMMAND, _packetData, count)) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_CUSTOM_COMMAND );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if (count) 
        {
            trace("%s(%i):WARNING(1): %s [ Antwortparameter (%i Byte) ]\n", __FILE__, __LINE__, __func__, count);
        }

        count = 0;

        if ((_result = call(IV_CONTEXT_SLAVE, IV_MODULE_APPLICATION, IV_FUNCTION_CUSTOM_COMMAND_OK, _packetData, count, IV_PENDING_TRIALS(getTimeout()))) != IV_ERROR_SUCCESS) 
        {
            // Fehler ausgeben
            traceError(deviceID, IV_MODULE_APPLICATION, IV_FUNCTION_CUSTOM_COMMAND_OK );
            return _result;
        }

        // Die Daten werden als eine Listen von Hexwerten mit \r getrennt geliefert.
        // Keine Antwortdaten erwartet

        // Daten wie sie sind übernehmen.
        int i = 0;
           for ( ; i < count && i < *sizeData; ++i )
               data[i] = _packetData[i];
           *sizeData = i;
    } 
    else 
    {
        unsigned char tmp[64];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_CUSTOM_COMMAND, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        // Aktuellen Status als aktive Antwort senden
        count = strlen( strcpy( (char*)tmp, "Keiner" ) ) + 1;

        // Aktive Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_MASTER, IV_MODULE_APPLICATION, IV_FUNCTION_CUSTOM_COMMAND_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
 * @brief Beschreibung fehlt noch.
 *
 * @param deviceID Anzusprechende Geraetenummer.
 */
int Channel4ModuleApplication::customCommandOK( short deviceID )
{
    trace("%s(%i):INFO(3): %s [ %i ]\n", __FILE__, __LINE__, __func__, deviceID);

    int count = 0;
    
    // Funktion in Remoteklasse aufrufen?
    if (getDeviceId() != deviceID)
    {
        trace("%s(%i):ERROR(1): %s \n", __FILE__, __LINE__, __func__);
    } 
    else 
    {
        unsigned char tmp[4];
        count = 0;
        
        // Antwort des Befehls senden.
        if ((_result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_APPLICATION, IV_FUNCTION_CUSTOM_COMMAND_OK, tmp, count)) != IV_ERROR_SUCCESS) 
        {	
            trace("%s(%i):ERROR(1): %s [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result);
            return _result;
        }

        trace("%s(%i):ERROR(1): %s [ %s ]\n", __FILE__, __LINE__, __func__, _result);
    }

    return _result;
}


