/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "Channel4ModuleHardware.h"
#include <stdio.h>

// ##################################################################
/**
* @brief Initialisieren der Membervariablen.
*
* @param id Nummer der Kanalklasse fuer z.B. Trace.
*/
Channel4ModuleHardware::Channel4ModuleHardware( short channelId, short deviceId )  : Channel4ProtocolCrypt( channelId, deviceId ) 
{

}

// ##################################################################
/**
* @brief Destruktor.
*
*/
Channel4ModuleHardware::~Channel4ModuleHardware() 
{
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Channel4ModuleHardware::getErrorModule( short module, short error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %p )\n", __FILE__, __LINE__, __func__, module, error, text.c_str() );

    // Ist IV_MODULE_HARDWARE ?
    if ( module == IV_MODULE_HARDWARE )
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ProtocolCrypt::getErrorModule( module, error, text );
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Channel4ModuleHardware::getErrorFunction( short module, short func, short error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %i, %p )\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str() );

    if ( module == IV_MODULE_HARDWARE )
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if ( error == 28 )
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %i ): Korrektes Passwort für die Kommunikation erforderlich.") % error );
            return;
        }
        else if ( error == 29 )
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %i ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt.") % error );
            return;
        }

        switch ( func )
        {
        case IV_FUNCTION_PING:
            // Setzen/Ermitteln des Geraetetyp
        case IV_FUNCTION_SET_TYPE_DEVICE:
        case IV_FUNCTION_GET_TYPE_DEVICE:

            // Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
        case IV_FUNCTION_SET_TYPE_BOARD:
        case IV_FUNCTION_GET_TYPE_BOARD:

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
        case IV_FUNCTION_SET_COMPONENT_RS485_0:
        case IV_FUNCTION_GET_COMPONENT_RS485_0:

            // Setzen/Ermitteln der Boardkomponente: TCP/IP
        case IV_FUNCTION_SET_TYPE_TCPIP:
        case IV_FUNCTION_GET_TYPE_TCPIP:

            // Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
        case IV_FUNCTION_SET_TYPE_MODEM_MC:
        case IV_FUNCTION_GET_TYPE_MODEM_MC:

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
        case IV_FUNCTION_SET_COMPONENT_RS485_1:
        case IV_FUNCTION_GET_COMPONENT_RS485_1:

            // Setzen/Ermitteln des Transpondertyps
        case IV_FUNCTION_SET_TYPE_TRANSPONDER:
        case IV_FUNCTION_GET_TYPE_TRANSPONDER:

            // Setzen/Ermitteln des Displaytyp
        case IV_FUNCTION_SET_TYPE_DISPLAY:
        case IV_FUNCTION_GET_TYPE_DISPLAY:

            // Setzen/Ermitteln des Tastaturtyps
        case IV_FUNCTION_SET_TYPE_KEYPAD:
        case IV_FUNCTION_GET_TYPE_KEYPAD:

            // Setzen/Ermitteln der Multimediakarte
        case IV_FUNCTION_SET_TYPE_MMC:
        case IV_FUNCTION_GET_TYPE_MMC:

            // Setzen/Ermitteln des GPS - Typs
        case IV_FUNCTION_SET_TYPE_GPS:
        case IV_FUNCTION_GET_TYPE_GPS:

            // Setzen/Ermitteln der USB - Komponente
        case IV_FUNCTION_SET_COMPONENT_USB:
        case IV_FUNCTION_GET_COMPONENT_USB:

            // Setzen/Ermitteln der Hardwareconfigurationsnummer
        case IV_FUNCTION_SET_HW_CONFIG:
        case IV_FUNCTION_GET_HW_CONFIG:

            // Setzen/Ermitteln des Funkmodultyps
        case IV_FUNCTION_SET_TYPE_RADIO:
        case IV_FUNCTION_GET_TYPE_RADIO:

            // Setzen/Ermitteln des Funkuhrmodultyps
        case IV_FUNCTION_SET_TYPE_DCF77:
        case IV_FUNCTION_GET_TYPE_DCF77:

            // Setzen/Ermitteln der analogen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_ANALOG:
        case IV_FUNCTION_GET_COMPONENT_IN_ANALOG:

            // Setzen/Ermitteln der digitalen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL:
        case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL:

            // Setzen/Ermitteln der digitalen Ausgaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL:
        case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL:

            // Setzen/Ermitteln des Fingerprint + Typs
        case IV_FUNCTION_SET_TYPE_FINGERPRINT:
        case IV_FUNCTION_GET_TYPE_FINGERPRINT:

        case IV_FUNCTION_SET_TYPE_EXTERNAL_READER:
        case IV_FUNCTION_GET_TYPE_EXTERNAL_READER:

            // Optionen schreiben/lesen
        case IV_FUNCTION_SET_OPTION_IOMODULE:
        case IV_FUNCTION_GET_OPTION_IOMODULE:

        case IV_FUNCTION_SET_OPTION_COMSERVER:
        case IV_FUNCTION_GET_OPTION_COMSERVER:

        case IV_FUNCTION_SET_OPTION_TBADAPTION:
        case IV_FUNCTION_GET_OPTION_TBADAPTION:
            text = boost::str( boost::format( "( %i ): Nicht eingetragener Funktionsfehler!") % error );
            break;

        default:
            text = boost::str( boost::format( "( %i ): Nicht eingetragener Fehlerblock für Funktion!") % error );
            break;
        }

        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Channel4ProtocolCrypt::getErrorFunction( module, func, error, text );
}

//#########################################################
//###################### HARDWARE #########################
//#########################################################

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleHardware::remoteCall( short module, short func, unsigned char *data, int &count )
{
    if ( module == IV_MODULE_HARDWARE )
    {
        switch ( func )
        {
        case IV_FUNCTION_PING:
            return ping( getDeviceId(), data, &count );

            // Setzen/Ermitteln des Geraetetyp
        case IV_FUNCTION_SET_TYPE_DEVICE:
            return setTypeDevice( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DEVICE:
            return getTypeDevice( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
        case IV_FUNCTION_SET_TYPE_BOARD:
            return setTypeBoard( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_BOARD:
            return getTypeBoard( getDeviceId(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
        case IV_FUNCTION_SET_COMPONENT_RS485_0:
            return setComponentRS485_0( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_RS485_0:
            return getComponentRS485_0( getDeviceId(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: TCP/IP
        case IV_FUNCTION_SET_TYPE_TCPIP:
            return setTypeTCPIP( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_TCPIP:
            return getTypeTCPIP( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
        case IV_FUNCTION_SET_TYPE_MODEM_MC:
            return setTypeModemMC( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_MODEM_MC:
            return getTypeModemMC( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
        case IV_FUNCTION_SET_COMPONENT_RS485_1:
            return setComponentRS485_1( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_RS485_1:
            return getComponentRS485_1( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln des Transpondertyps
        case IV_FUNCTION_SET_TYPE_TRANSPONDER:
            return setTypeTransponder( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_TRANSPONDER:
            return getTypeTransponder( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln des Displaytyp
        case IV_FUNCTION_SET_TYPE_DISPLAY:
            return setTypeDisplay( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DISPLAY:
            return getTypeDisplay( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln des Tastaturtyps
        case IV_FUNCTION_SET_TYPE_KEYPAD:
            return setTypeKeypad( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_KEYPAD:
            return getTypeKeypad( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln der Multimediakarte
        case IV_FUNCTION_SET_TYPE_MMC:
            return setTypeMMC( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_MMC:
            return getTypeMMC( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln des GPS - Typs
        case IV_FUNCTION_SET_TYPE_GPS:
            return setTypeGPS( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_GPS:
            return getTypeGPS( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln der USB - Komponente
        case IV_FUNCTION_SET_COMPONENT_USB:
            return setComponentUSB( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_USB:
            return getComponentUSB( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln der Hardwareconfigurationsnummer
        case IV_FUNCTION_SET_HW_CONFIG:
            return setHWConfig( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_HW_CONFIG:
            return getHWConfig( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln des Funkmodultyps
        case IV_FUNCTION_SET_TYPE_RADIO:
            return setTypeRadio( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_RADIO:
            return getTypeRadio( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln des Funkuhrmodultyps
        case IV_FUNCTION_SET_TYPE_DCF77:
            return setTypeDCF77( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DCF77:
            return getTypeDCF77( getDeviceId(), (unsigned char&)data[0] ); 

            // Setzen/Ermitteln der analogen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_ANALOG:
            return setComponentInAnalog( getDeviceId(), *(unsigned int*)data, *(unsigned short*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_IN_ANALOG:
            return getComponentInAnalog( getDeviceId(), (unsigned int&)data[0] ); 

            // Setzen/Ermitteln der digitalen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL:
            return setComponentInDigital( getDeviceId(), *(unsigned int*)data, *(unsigned short*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL:
            return getComponentInDigital( getDeviceId(), (unsigned int&)data[0] ); 

            // Setzen/Ermitteln der digitalen Ausgaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL:
            return setComponentOutDigital( getDeviceId(), *(unsigned int*)data, *(unsigned short*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL:
            return getComponentOutDigital( getDeviceId(), (unsigned int&)data[0] );

            // Setzen/Ermitteln des Fingerprint + Typs
        case IV_FUNCTION_SET_TYPE_FINGERPRINT:
            return setTypeFingerprint( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_FINGERPRINT:
            return getTypeFingerprint( getDeviceId(), (unsigned char&)data[0] );

        case IV_FUNCTION_SET_TYPE_EXTERNAL_READER:
            return setTypeExternalReader( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_EXTERNAL_READER:
            return getTypeExternalReader( getDeviceId(), (unsigned char&)data[0] );

            // Optionen schreiben/lesen
        case IV_FUNCTION_SET_OPTION_IOMODULE:
            return setOptionIOModule( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_OPTION_IOMODULE:
            return getOptionIOModule( getDeviceId(), (unsigned char&)data[0] );

        case IV_FUNCTION_SET_OPTION_COMSERVER:
            return setOptionComServer( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_OPTION_COMSERVER:
            return getOptionComServer( getDeviceId(), (unsigned char&)data[0] );

        case IV_FUNCTION_SET_OPTION_TBADAPTION:
            return setOptionTBAdaption( getDeviceId(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_OPTION_TBADAPTION:
            return getOptionTBAdaption( getDeviceId(), (unsigned char&)data[0] );

            // Kommando fuer Pruefplatz
        case IV_FUNCTION_INSPECTION_STATION_OK:
            return commandInspectionStationOK( getDeviceId() );

            // Funktion nicht definiert
        default:
            return 0x0200;
        }
    }

    return Channel4ProtocolCrypt::remoteCall( module, func, data, count );
}

// ##################################################################
/**
* @brief Ermitteln des Geraetetyps.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Geraetetyp
*             0 = PZE - Master IV
*             1 = Zutritts - Master IV
*             2 = BDE - Master IV
*           ... = ...
*/
int Channel4ModuleHardware::getTypeDevice( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, &_hwInfo.typeDevice, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die vorliegende Board-Revisionsnummer.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Revisionsnummer in der Form (HiNibble Majornummer).(LoNibble Minornummer)
*             z. B. 0x14 -> Major 1, Minor (Redesign Nr.) 4
*/
int Channel4ModuleHardware::getTypeBoard(short deviceId, unsigned char &val)
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, &_hwInfo.typeBoard, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentRS485_0(short deviceId, unsigned char &val)
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0 );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, &_hwInfo.componentRS485_0, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Keines
*              1 = MC35i
*              2 = MC55
*            ... = ...
*/
int Channel4ModuleHardware::getTypeTCPIP( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, &_hwInfo.typeTCPIP, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Keines
*              1 = MC35i
*              2 = MC55
*            ... = ...
*/
int Channel4ModuleHardware::getTypeModemMC( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, &_hwInfo.typeModemMC, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentRS485_1( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1 );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, &_hwInfo.componentRS485_1, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keines
*             1 = Lehner kleines
*             2 = Lehner grosses
*           ... = ...
*/
int Channel4ModuleHardware::getTypeDisplay( short deviceId, unsigned char &val, int checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, &_hwInfo.typeDisplay, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = PZE - Master IV
*             2 = BDE - Master IV
*           ... = ...
*/
int Channel4ModuleHardware::getTypeKeypad( short deviceId, unsigned char &val, int checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, &_hwInfo.typeKeypad, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keiner
*             1 = TSR30
*             2 = TSR32
*             3 = PHG PRIMO 100
*             4 = Cavitec Mifare
*             5 = Dallas iButton
*           ... = ...
*/
int Channel4ModuleHardware::getTypeTransponder( short deviceId, unsigned char &val, int checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, &_hwInfo.typeTransponder, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Fest integriert
*             2 = Herausnehmbar
*           ... = ...
*/
int Channel4ModuleHardware::getTypeMMC( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, &_hwInfo.typeMMC, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Bluegiga (Bluetooth)
*           ... = ...
*/
int Channel4ModuleHardware::getTypeRadio( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, &_hwInfo.typeRadio, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Furuno GN-80
*            ... = ...
*/
int Channel4ModuleHardware::getTypeGPS( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, &_hwInfo.typeGPS, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentUSB( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, &_hwInfo.componentUSB, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentBattery( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, &_hwInfo.componentBattery, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentCharge( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, &_hwInfo.componentCharge, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = HKW
*             4 = Nicht zu verwenden, da auf 0 gemappt.
*           ... = ...
*/
int Channel4ModuleHardware::getTypeDCF77( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77 );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        // Wert wurde von FWV 04.01.01 fuer die Baudrate von Bluetooth verwendet und
        // steht nach initialisierung auf dem Wert 4. Dieser Wert, wird als Typ ignoriert.
        val = (_packetData[0] != 0xFF && _packetData[0] != 4) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, &_hwInfo.typeDCF77, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentInAnalog( short deviceId, unsigned int &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned int) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (*(unsigned int*)_packetData != 0xFFFFFFFF) ? *(unsigned int*)_packetData : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, (unsigned char*)&_hwInfo.componentInAnalog, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentInDigital( short deviceId, unsigned int &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned int) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (*(unsigned int*)_packetData != 0xFFFFFFFF) ? *(unsigned int*)_packetData : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, (unsigned char*)&_hwInfo.componentInDigital, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getComponentOutDigital( short deviceId, unsigned int &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned int) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (*(unsigned int*)_packetData != 0xFFFFFFFF) ? *(unsigned int*)_packetData : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, (unsigned char*)&_hwInfo.componentOutDigital, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getOptionIOModule( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = _packetData[0];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, &_hwInfo.optionIOModule, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getOptionComServer( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = _packetData[0];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, &_hwInfo.optionComserver, count )) != IV_ERROR_SUCCESS )
        {	
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }

        return 0x0200;
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getOptionTBAdaption( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = _packetData[0];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, &_hwInfo.optionTBAdaption, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleHardware::commandInspectionStation( short deviceId, unsigned char address, char *data, int sizeData, unsigned char &addressAnswer, char *dataAnswer, int &sizeAnswer, int toPending )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %p, %i )\n", __FILE__, __LINE__, __func__, deviceId, address, data, sizeData );

    int count = 0, i;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Parameter eintragen
        // Erster Parameter ist die Adresse
        _packetData[0] = address;
        count = sizeof(unsigned char);
        // Folgender Parameter ist eine Optionale Zeichenkette
        for ( i = 0; i < sizeData; ++i )
            _packetData[count + i] = data[i];

        count += sizeData;
        // Nullterminierung sicherstellen
        if ( _packetData[count] != '\0' )
        {
            count++;
            _packetData[count] = '\0';
        }

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION );
            return _result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (_result = call( IV_CONTEXT_SLAVE, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, _packetData, count, ((toPending*1000) / getTimeout()) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION );
            return _result;
        }

        // Erstes Byte ist die Adresse
        addressAnswer = _packetData[0];
        // Folgend ein optionaler String
        for ( i = 0; i < (count-1) && i < sizeAnswer; ++i )
            dataAnswer[i] = _packetData[1 + i];

        // Laenge der Zeichenkette
        sizeAnswer = i;
    }
    else 
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceId Anzusprechende Geraetenummer.
*/
int Channel4ModuleHardware::commandInspectionStationOK( short deviceId )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, deviceId );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        trace( "%s(%i):ERROR(1): %s()\n", __FILE__, __LINE__, __func__ );
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (_result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, _result );
            return _result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getOptionTPWrite( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = _packetData[0];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, &_hwInfo.optionTPWrite, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getOptionHTTP( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = _packetData[0];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, &_hwInfo.optionHttp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getOptionLanguage( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = _packetData[0];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, &_hwInfo.optionLanguage, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getOptionBiokeyUpdate( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = _packetData[0];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, &_hwInfo.optionBiokeyUpdate, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die bezahlbaren Optionen enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param mask Maske, welche Optionen unterstuetzt werden.
* @param options Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Channel4ModuleHardware::getPayableOptions( short deviceId, unsigned char &mask, unsigned char &options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%02X, 0x%02X )\n", __FILE__, __LINE__, __func__, deviceId, mask, options );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != (2 * sizeof(unsigned char)) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            mask = 0;
            options = 0;
            return _result;
        }

        // Werte uebernehmen
        mask = _packetData[0];
        options = _packetData[1];
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = 2 * sizeof(unsigned char);
        unsigned char buf[2];
        buf[0] = 0x01;
        buf[1] = _hwInfo.optionsPayable;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, buf, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Idencom (BioKey 3000)
*           ... = ...
*/
int Channel4ModuleHardware::getTypeFingerprint( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, &_hwInfo.typeFingerprint, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Bitfeld der verwendbaren Varianten.
*            -1 = Undefiniert
*            Bit 1 = GIS
*            Bit 2 = Protect
*            Bit 3 = PHG
*           ... = ...
*/
int Channel4ModuleHardware::getTypeExternalReader( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // BUGFIX
        // Eine Firmware die die Serien TS und Protect unterstützt setzt als unterstützte Serien
        // TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
        // so dass der Wert der Hardwareinfo eindeutig ist.

        // Wert uebernehmen
        val = _packetData[0];

        // Werden beide Flags vorgefunden wird von TS alleine ausgegangen.
        if ( (val != 0xFF) && (val & IV_VALUE_TYPE_EXTERNAL_READER_GIS) && (val & IV_VALUE_TYPE_EXTERNAL_READER_PROTECT) )
        {
            val = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
        }
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, &_hwInfo.typeExternalReader, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Exklusiv-Line IV
*              2 = Flex-MasterIV
*            ... = ...
*/
int Channel4ModuleHardware::getTypeTouch( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, &_hwInfo.typeTouch, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Vibration
*              2 = Beschleunigung
*            ... = ...
*/
int Channel4ModuleHardware::getTypeSensor( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, &_hwInfo.typeSensor, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Setzen der Optionen fuer Setup.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param mask Gibt die gültigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int Channel4ModuleHardware::setSetupOptions( short deviceId, unsigned short mask, unsigned short options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%04X, 0x%04X )\n", __FILE__, __LINE__, __func__, deviceId, mask, options );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        *(unsigned short*)&_packetData[0] = mask;
        count += sizeof(unsigned short);

        *(unsigned short*)&_packetData[2] = options;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = 0;

        unsigned char val = 0;
        // Wert an interne Struktur uebernehmen
        _hwInfo.optionsSetup = options;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermitteln der Optionen fuer Setup.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param mask Gibt die gültigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int Channel4ModuleHardware::getSetupOptions( short deviceId, unsigned short &mask, unsigned short &options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%04X, 0x%04X )\n", __FILE__, __LINE__, __func__, deviceId, mask, options );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count != (2 * sizeof(unsigned short)) )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Die Komponendenangabe uebernehmen.
        mask = *(unsigned short*)&_packetData[0];
        options = *(unsigned short*)&_packetData[2];
    }
    else 
    {
        int result;
        // Parametergroesse setzen
        count = 0;

        unsigned char val;
        // Wert an interne Struktur uebernehmen
        options = _hwInfo.optionsSetup;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die gesamte Hardwareinfo in einer einzelnen Struktur.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param ebs Struktur mit Informationen zur Hardware.
*/
int Channel4ModuleHardware::getHardwareInfo( short deviceId, StructHardwareInfo &hwInfo )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, deviceId );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count < (int)sizeof(StructHardwareInfo) )
        {
            // Wenn nur die Bezahloptionen fehlen, dann trotzdem annehmen.
            // Uebergang von 04.02.02 auf 04.02.03 (29.05.2012)
            if (count != 0x2b)
                return 0x0300; // Es wurden weniger Daten gelesen als aktuell bekannt.

            _packetData[count - 1] = 0; // Es fehlen noch die Bezahloptionen
            count++;
                
        }
        else if ( count > (int)sizeof(StructHardwareInfo) )
        {
            // Es wurden mehr Daten gelesen als aktuell bekannt.
            return 0x0400;
            // Fehler 4
        }

        // Werte uebernehmen
        hwInfo = *((StructHardwareInfo*)&_packetData[0]);
    }
    else
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(StructHardwareInfo);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO, (unsigned char*)&hwInfo, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

// ##################################################################
/**
* @brief Ermittelt die "Version" der Hardwareconfiguration.
*
* @param deviceId Anzusprechende Geraetenummer.
* @param val Wert fuer Version der Hardwareconfiguration.
*/
int Channel4ModuleHardware::getHWConfig( short deviceId, unsigned char &val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( deviceId, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG );
            return _result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return _result;
        }

        // Wert uebernehmen
        val = (_packetData[0] != 0xFF) ? _packetData[0] : 0;
    }
    else 
    {
        int result;
        // Parametergroesse setzen
        count = sizeof(unsigned char);

        unsigned char tmp[1];
        tmp[0] = 1;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, tmp, count )) != IV_ERROR_SUCCESS )
        {			
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}


#if !defined( DATAFOX_STUDIO_IV ) && !defined( DF_API_PRODUCTION )
// Funktions-Stumps

int Channel4ModuleHardware::pingRandom( short deviceId )
{ 
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Channel4ModuleHardware::setTypeDevice( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeBoard( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeTCPIP( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeModemMC( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeDisplay( short deviceId, unsigned char val, unsigned short val2, int checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeKeypad( short deviceId, unsigned char val, unsigned short val2, int checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeTransponder( short deviceId, unsigned char val, unsigned short val2, int checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeMMC( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeRadio( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeGPS( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeDCF77( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeFingerprint( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeExternalReader( short deviceId, unsigned char val, unsigned short val2 ) 
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;
    unsigned short key;
    switch ( val ) 
    {
    case 0xFF:
        key = 0xCF38;
        break;

    case 0x00:
        key = 0x8F78;
        break;

    case 0x01:
        key = 0x4FB9;
        break;

    case 0x02:
        key = 0x4EF9;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        _packetData[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&_packetData[1] = key;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        _hwInfo.optionBiokeyUpdate = val;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}
int Channel4ModuleHardware::setTypeTouch( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setTypeSensor( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}


int Channel4ModuleHardware::setComponentRS485_0( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setComponentRS485_1( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setComponentUSB( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setComponentInAnalog( short deviceId, unsigned int val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setComponentInDigital( short deviceId, unsigned int val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setComponentOutDigital( short deviceId, unsigned int val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setComponentBattery( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setComponentCharge( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId ) 
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Channel4ModuleHardware::setOptionIOModule( short deviceId, unsigned char val, unsigned short val2 )     
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;
    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0C59;
        break;

    case 0x00:
        key = 0x4C19;
        break;

    case 0x01:
        key = 0x8CD8;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        _packetData[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&_packetData[1] = key;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        _hwInfo.optionIOModule = val;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

int Channel4ModuleHardware::setOptionComServer( short deviceId, unsigned char val, unsigned short val2 )    
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;
    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCCF8;
        break;

    case 0x00:
        key = 0x8CB8;
        break;

    case 0x01:
        key = 0x4C79;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        _packetData[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&_packetData[1] = key;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        _hwInfo.optionComserver = val;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

int Channel4ModuleHardware::setOptionTBAdaption( short deviceId, unsigned char val, unsigned short val2 )   
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;
    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCD18;
        break;

    case 0x00:
        key = 0x8D58;
        break;

    case 0x01:
        key = 0x4D99;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        _packetData[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&_packetData[1] = key;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        _hwInfo.optionTBAdaption = val;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

int Channel4ModuleHardware::setOptionTPWrite( short deviceId, unsigned char val, unsigned short val2 )   
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );

    int count = 0;
    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0BD9;
        break;

    case 0x00:
        key = 0x4B99;
        break;

    case 0x01:
        key = 0x8B58;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }
    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        _packetData[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&_packetData[1] = key;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
        // Schluessel pruefen: Funktionsfehler
        if (key != val2) return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        _hwInfo.optionTPWrite = val;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

int Channel4ModuleHardware::setOptionHTTP( short deviceId, unsigned char val, unsigned short val2  )   
{ 
    trace("%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val);

    int count = 0;
    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCB78;
        break;

    case 0x00:
        key = 0x8B38;
        break;

    case 0x01:
        key = 0x4BF9;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        _packetData[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&_packetData[1] = key;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
        // Schluessel pruefen: Funktionsfehler
        if (key != val2) return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        _hwInfo.optionHttp = val;	

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, &val, count)) != IV_ERROR_SUCCESS)
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return _result;
}

int Channel4ModuleHardware::setOptionLanguage( short deviceId, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setOptionBiokeyUpdate( short deviceId, unsigned char val, unsigned short val2 )   
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, deviceId, val );
    int count = 0;
    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0A39;
        break;

    case 0x00:
        key = 0x4A79;
        break;

    case 0x01:
        key = 0x8AB8;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( getDeviceId() != deviceId )
    {
        // Umgebung zuruecksetzen
        reset();

        // Remoteadresse setzen.
        setDeviceIdRemote( deviceId );

        // Die Komponendenangabe uebernehmen.
        _packetData[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&_packetData[1] = key;
        count += sizeof(unsigned short);

        if ( (_result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, _packetData, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            traceError( getDeviceIdRemote(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE );
            return _result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        _hwInfo.optionBiokeyUpdate = val;	

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
    }

    return _result;
}
int Channel4ModuleHardware::setPayableOptions( short deviceId, unsigned char mask, unsigned char options, unsigned short val2 )
{ 
    (void)mask; 
    (void)options; 
    (void)val2; 
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Channel4ModuleHardware::setHWConfig( short deviceId, unsigned char val, unsigned short val2 )
{
    (void)val;
    (void)val2;
    if ( getDeviceId() != deviceId )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}
#endif

